@@ -67,6 +67,8 @@ class MetadataBlock(object):
         codes[-1][0] |= 128
         for code, datum in codes:
             byte = chr(code)
+            if len(datum) > 2**24:
+                raise error("block is too long to write")
             length = struct.pack(">I", len(datum))[-3:]
             data.append(byte + length + datum)
         return "".join(data)
@@ -482,7 +484,13 @@ class Padding(MetadataBlock):
 
     def __init__(self, data=""): super(Padding, self).__init__(data)
     def load(self, data): self.length = len(data.read())
-    def write(self): return "\x00" * self.length
+    def write(self):
+        try: return "\x00" * self.length
+        # On 64 bit platforms this won't generate a MemoryError
+        # or OverflowError since you might have enough RAM, but it
+        # still generates a ValueError.
+        except (OverflowError, ValueError, MemoryError):
+            raise error("cannot write %d bytes" % self.length)
     def __eq__(self, other):
         return isinstance(other, Padding) and self.length == other.length
     def __repr__(self):
@@ -514,6 +522,9 @@ class FLAC(FileType):
         size = to_int_be(file.read(3))
         try:
             data = file.read(size)
+            if len(data) != size:
+                raise error(
+                    "file said %d bytes, read %d bytes" % (size, len(data)))
             block = self.METADATA_BLOCKS[byte & 0x7F](data)
         except (IndexError, TypeError):
             block = MetadataBlock(data)
@@ -563,8 +574,8 @@ class FLAC(FileType):
         self.cuesheet = None
         self.seektable = None
         self.filename = filename
+        fileobj = file(filename, "rb")
         try:
-            fileobj = file(filename, "rb")
             self.__check_header(fileobj)
             while self.__read_metadata_block(fileobj): pass
         finally:
