@@ -378,6 +378,8 @@ def TestReadTags():
     tests = [
     ['TALB', '\x00a/b', 'a/b', '', dict(encoding=0)],
     ['TBPM', '\x00120', '120', 120, dict(encoding=0)],
+    ['TCMP', '\x001', '1', 1, dict(encoding=0)],
+    ['TCMP', '\x000', '0', 0, dict(encoding=0)],
     ['TCOM', '\x00a/b', 'a/b', '', dict(encoding=0)],
     ['TCON', '\x00(21)Disco', '(21)Disco', '', dict(encoding=0)],
     ['TCOP', '\x001900 c', '1900 c', '', dict(encoding=0)],
@@ -593,6 +595,8 @@ def TestReadTags():
     ['TMT', '\x00DIG/A', 'DIG/A', '', dict(encoding=0)],
     ['TFT', '\x00MPG/3', 'MPG/3', '', dict(encoding=0)],
     ['TBP', '\x00133', '133', 133, dict(encoding=0)],
+    ['TCP', '\x001', '1', 1, dict(encoding=0)],
+    ['TCP', '\x000', '0', 0, dict(encoding=0)],
     ['TCR', '\x00Me', 'Me', '', dict(encoding=0)],
     ['TPB', '\x00Him', 'Him', '', dict(encoding=0)],
     ['TEN', '\x00Lamer', 'Lamer', '', dict(encoding=0)],
@@ -1463,6 +1467,21 @@ class WriteRoundtrip(TestCase):
         ID3(self.newsilence).delete()
         self.assertEquals(open(self.newsilence).read(10), 'abc')
 
+    def test_frame_order(self):
+        from mutagen.id3 import TIT2, APIC, TALB, COMM
+        f = ID3(self.newsilence)
+        f["TIT2"] = TIT2(encoding=0, text="A title!")
+        f["APIC"] = APIC(encoding=0, mime="b", type=3, desc='', data="a")
+        f["TALB"] = TALB(encoding=0, text="c")
+        f["COMM"] = COMM(encoding=0, desc="x", text="y")
+        f.save()
+        data = open(self.newsilence, 'rb').read()
+        self.assert_(data.find("TIT2") < data.find("APIC"))
+        self.assert_(data.find("TIT2") < data.find("COMM"))
+        self.assert_(data.find("TALB") < data.find("APIC"))
+        self.assert_(data.find("TALB") < data.find("COMM"))
+        self.assert_(data.find("TIT2") < data.find("TALB"))
+
     def tearDown(self):
         try: os.unlink(self.newsilence)
         except EnvironmentError: pass
@@ -1538,6 +1557,21 @@ class FrameIDValidate(TestCase):
         self.failIf(is_valid_frame_id("MP3e"))
         self.failIf(is_valid_frame_id("+ABC"))
 
+class BadTYER(TestCase):
+    uses_mmap = False
+    filename = join('tests', 'data', 'bad-TYER-frame.mp3')
+    def setUp(self):
+        self.audio = ID3(self.filename)
+
+    def test_no_year(self):
+        self.failIf("TYER" in self.audio)
+
+    def test_has_title(self):
+        self.failUnless("TIT2" in self.audio)
+
+    def tearDown(self):
+        del(self.audio)
+
 add(ID3Loading)
 add(ID3GetSetDel)
 add(BitPaddedIntTest)
@@ -1553,6 +1587,7 @@ add(WriteRoundtrip)
 add(OddWrites)
 add(NoHash)
 add(FrameIDValidate)
+add(BadTYER)
 
 try: import eyeD3
 except ImportError: pass
